<template>
  <b-card-code>
    <b-form @submit.prevent="onSubmit">
      <!-- 全優惠模組共通表單欄位組件 -->
      <promotion-base-form
        :promotion="promotion"
        :strErrors="strErrors"
        @initChannels="initChannels"
      ></promotion-base-form>
      <!-- 全優惠模組共通表單欄位組件 End -->

      <b-row>
        <b-col cols="12">
          <b-form-group
            label="*優惠代碼"
            label-cols-md="1"
            class="text-left"
            label-size="lg"
          >
            <b-form-input
              type="text"
              v-model="promotion.code"
              placeholder="請輸入優惠代碼"
            />
          </b-form-group>
        </b-col>

        <b-col cols="12">
          <b-form-group
            label="優惠代碼內容"
            label-cols-md="1"
            class="text-left"
            label-size="lg"
          >
            <b-button
              class="d-inline-block"
              label-cols-md="1"
              variant="primary"
              @click="addContent"
            >
              增加內容
            </b-button>
          </b-form-group>
        </b-col>

        <b-col cols="11" offset-md="1">
          <div ref="form" class="repeater-form">
            <div v-for="(content, index) in promotion.contents" :key="index">
              <b-row class="d-flex rounded mb-1">
                <b-col cols="12" md="2">
                  <label class="d-inline">內容類型</label>
                  <v-select
                    v-model="content.type"
                    :options="contentTypeOptions"
                    :clearable="false"
                    :reduce="(type) => type.id"
                    label="name"
                    class="item-selector-title"
                  />
                </b-col>

                <b-col cols="12" md="4" v-if="content.type == 1">
                  <label class="d-inline">折價金額</label>
                  <b-form-input
                    v-model="content.discount_amount"
                    type="number"
                    class=""
                    placeholder="請輸入折價金額"
                  />
                </b-col>

                <b-col cols="12" md="4" v-if="content.type == 2">
                  <label class="d-inline">商品</label>
                  <v-select
                    v-model="content.product_id"
                    :options="products"
                    :clearable="true"
                    :reduce="(product) => product.id"
                    label="name_frontend"
                    class="item-selector-title"
                  />
                </b-col>

                <b-col cols="12" md="auto">
                  <label class="d-inline">是否啟用</label>
                  <div style="height: 32px; padding-top: 6px">
                    <b-form-checkbox
                      v-model="content.enabled"
                      checked="true"
                      class="custom-control-primary"
                      name="check-button"
                      switch
                    />
                  </div>
                </b-col>

                <b-col cols="12" md="auto">
                  <label class="d-inline">刪除</label>
                  <div>
                    <feather-icon
                      size="32"
                      icon="XIcon"
                      class="cursor-pointer text-danger"
                      @click="removeContent(index)"
                    />
                  </div>
                </b-col>
              </b-row>
            </div>
          </div>
        </b-col>

        <b-col
          cols="12"
          v-for="channel in promotion.channels"
          :key="channel.channel_id"
        >
          <b-form-group
            :label="`${channel.channel_name}可使用數`"
            label-cols-md="1"
            class="text-left"
            label-size="lg"
          >
            <b-form-input v-model="channel.used_limit" min="0" />
          </b-form-group>
        </b-col>

        <b-col cols="12">
          <b-form-group
            label="總可使用次數"
            label-cols-md="1"
            class="text-left"
            label-size="lg"
          >
            <b-form-input
              type="number"
              v-model="totalUsedLimit"
              placeholder="請輸入可使用數"
              disabled="disabled"
            />
          </b-form-group>
        </b-col>
        <!-- submit and reset -->
      </b-row>

      <b-row align-h="end">
        <b-col cols="auto">
          <b-button
            class="float-right"
            v-ripple.400="'rgba(255, 255, 255, 0.15)'"
            type="submit"
            variant="primary"
          >
            {{ promotion.id ? "儲存" : "新增" }}
          </b-button>
        </b-col>
      </b-row>
    </b-form>
  </b-card-code>
</template>

<script>
import BCardCode from "@core/components/b-card-code";
import Ripple from "vue-ripple-directive";
import vSelect from "vue-select";
import store from "@/store";
import { onUnmounted, ref } from "@vue/composition-api";
import router from "@/router";
import PromotionBaseForm from "../PromotionBaseForm.vue";
import promotionModule from "../promotionModule";

export default {
  components: {
    BCardCode,
    vSelect,
    PromotionBaseForm,
  },
  directives: {
    Ripple,
  },
  watch: {
    "promotion.apply_channels": {
      handler: function (apply_channels) {
        this.channels.forEach((channel) => {
          let isSelected = apply_channels.some(
            (apply_channel) => apply_channel.id == channel.id
          );

          let has = this.promotion.channels.some(
            (promotionChannel) => promotionChannel.channel_id == channel.id
          );

          // 新增promotion.channels
          if (isSelected && !has) {
            this.promotion.channels.push({
              channel_id: channel.id,
              channel_name: channel.name,
              used_limit: 0,
            });
          }

          // 移除promotion.channels
          if (!isSelected && has) {
            this.promotion.channels = this.promotion.channels.filter(
              (promotionChannel) => promotionChannel.channel_id != channel.id
            );
          }
        });

        this.promotion.channels.sort((a, b) => a.channel_id - b.channel_id);
      },
    },
  },
  computed: {
    totalUsedLimit() {
      return this.promotion.channels.reduce(
        (total, channel) => total + parseInt(channel.used_limit),
        0
      );
    },
  },
  mounted() {
    if (router.currentRoute.params.id) {
      this.getPromotion();
    }

    this.getProducts();
  },
  methods: {
    // 刪除條件
    removeContent(index) {
      this.promotion.contents.splice(index, 1);
    },

    // 增加條件
    addContent() {
      this.promotion.contents.push({
        type: 1,
        value: null,
        discount_amount: null,
        is_accumulate: true,
        enabled: true,
      });
    },

    // 新增資料
    onSubmit() {
      if (router.currentRoute.params.id) {
        this.updatePromotion();
      } else {
        this.createPromotion();
      }
    },

    // 新增優惠
    createPromotion() {
      store
        .dispatch("promotions/createPromotion", {
          typeName: this.TYPE_NAME,
          promotion: this.promotion,
        })
        .then((response) => {
          let result = response.data;

          // 錯誤處理
          if (result.success === false) {
            this.errorMessage(result.message);

            return false;
          }

          this.successMessage("新增成功");

          router.push({ name: "promotions-promo-codes-list" });
        })
        .catch((error) => {
          let result = error.response;

          if (result.status === 422 && result?.data?.errors) {
            this.strErrors = result.data.errors;
          }

          this.errorMessage("新增失敗：" + result?.data?.message);
        });
    },

    // 更新優惠
    updatePromotion() {
      store
        .dispatch("promotions/updatePromotion", {
          typeName: this.TYPE_NAME,
          promotion: this.promotion,
        })
        .then((response) => {
          let result = response.data;

          // 錯誤處理
          if (result.success === false) {
            this.errorMessage(result.message);

            return false;
          }

          this.successMessage("更新成功");

          router.push({ name: "promotions-promo-codes-list" });
        })
        .catch((error) => {
          let result = error.response;

          if (result.status === 422 && result?.data?.errors) {
            this.strErrors = result.data.errors;
          }

          this.errorMessage("更新失敗：" + result?.data?.message);
        });
    },

    // 取得優惠
    getPromotion() {
      store
        .dispatch("promotions/getPromotion", {
          typeName: this.TYPE_NAME,
          id: router.currentRoute.params.id,
        })
        .then((response) => {
          if (response.data?.success === false) {
            this.errorMessage(response.data?.message);

            return false;
          }

          this.promotion = response.data?.data;
        })
        .catch((error) => {
          this.errorMessage(error.response?.data?.message);
        });
    },

    // 取得產品清單
    getProducts() {
      store
        .dispatch("promotions/getProducts", {})
        .then((response) => {
          if (response.data?.success === false) {
            this.errorMessage(response.data?.message);

            return false;
          }

          this.products = response.data?.data;
        })
        .catch((error) => {
          this.errorMessage(error.response?.data?.message);
        });
    },

    // 取得渠道
    initChannels(channels = []) {
      this.channels = channels;
    },

    // 顯示成功視窗
    successMessage(text = null, title = "成功") {
      this.$swal({
        title: title,
        text: text,
        icon: "success",
        customClass: {
          confirmButton: "btn btn-primary",
        },
        buttonsStyling: false,
      });
    },

    // 顯示錯誤視窗
    errorMessage(text = null, title = "錯誤") {
      this.$swal({
        title: title,
        text: text,
        icon: "error",
        customClass: {
          confirmButton: "btn btn-primary",
        },
        buttonsStyling: false,
      });
    },
  },
  setup() {
    const USER_APP_STORE_MODULE_NAME = "promotions";

    // Register module
    if (!store.hasModule(USER_APP_STORE_MODULE_NAME))
      store.registerModule(USER_APP_STORE_MODULE_NAME, promotionModule);

    // UnRegister on leave
    onUnmounted(() => {
      if (store.hasModule(USER_APP_STORE_MODULE_NAME))
        store.unregisterModule(USER_APP_STORE_MODULE_NAME);
    });

    // 優惠類型
    const TYPE_NAME = "promo_codes";

    let contentTypeOptions = ref([
      { id: 1, name: "現金" },
      { id: 2, name: "商品" },
    ]);

    let promotion = ref({
      id: null,
      name: null,
      start_at: null,
      end_at: null,
      description: null,
      excepts: [],
      apply_channels: [],
      type: 5, // 預設優惠類型，很重要，影響到新增
      contents: [],
      channels: [],
    });

    let strErrors = ref({
      name: null,
      start_at: null,
      end_at: null,
      description: null,
      excepts: null,
      apply_channels: null,
      type: null,
      contents: null,
      channels: null,
    });

    let channels = ref([]);

    let products = ref([]);

    return {
      TYPE_NAME,
      contentTypeOptions,
      promotion,
      strErrors,
      channels,
      products,
    };
  },
};
</script>
<style lang="scss">
@import "~@resources/scss/vue/libs/quill.scss";
@import "~@resources/scss/vue/libs/vue-sweetalert.scss";
@import "~@resources/scss/vue/libs/vue-select.scss";
</style>